#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "GrowthFunction.h"
#include "LinearPS.h"
#include "Math.h"


LinearPS::LinearPS(const Cosmology& C_, real z_) : C(C_) {
    z = z_;
    GrowthFunction D(C);

    /* Get transfer function */
    const array& k = C.ki;
    const array& T = C.Ti;

    /* Calculate P(k) */
    int N = k.size();
    array P(N);
    for(int i = 0; i < N; i++)
        P[i] = 2*M_PI*M_PI/pow3(k[i]) * pow2(C.delta_H) * pow(2997.925*k[i], 3+C.n) * pow2(T[i]) * pow2(D(z)/D(0));
    Pinterp = InterpolatedPS(C, k, P);

    kr = k.back();
    pr = P.back();
}

real LinearPS::Evaluate(real k) const {
    if(k < kr)
        return Pinterp(k);
    else {
        real omh = C.Omega_m*C.h;
        return pr * pow2(log(1.8*k/omh))*pow(k, C.n-4) / pow2(log(1.8*kr/omh))*pow(kr, 4-C.n);
    }
}
