#ifndef LINEAR_PS_H
#define LINEAR_PS_H

#include "Common.h"
#include "PowerSpectrum.h"
#include "InterpolatedPS.h"


/**********************************************************************
 * LinearPS
 *
 * Linear power spectrum interpolated from a transfer function, which is
 * presumably calculated from a Boltzmann code.  Uses the analytic
 * $\ln^2(k)/k^3$ formula at high $k$.
 *********************************************************************/
class LinearPS : public PowerSpectrum {
public:
    LinearPS(const Cosmology& C, real z = 0);

    const Cosmology& GetCosmology() const { return C; }
    real Evaluate(real k) const;

private:
    const Cosmology& C;
    real z;
    InterpolatedPS Pinterp;
    real kr, pr;        // k and P(k) of right-most data point
};

#endif // LINEAR_PS_H
