#if HAVE_CONFIG_H
# include <config.h>
#endif

#include <gsl/gsl_sf_bessel.h>
#include <gsl/gsl_sf_gamma.h>

#include "Math.h"

namespace SpecialFunctions {

/* Cylindrical Bessel functions */
double J_0(double x) { return gsl_sf_bessel_J0(x); }
double J_1(double x) { return gsl_sf_bessel_J1(x); }

/* Spherical Bessel functions */
double j_0(double x) { return gsl_sf_bessel_j0(x); }
double j_1(double x) { return gsl_sf_bessel_j1(x); }

/* Unnormalized upper incomplete gamma function:
 *  $\Gamma(a,x) = \int_x^\infty t^{a-1} e^{-t} dt$ */
double upper_gamma(double a, double x) {
    return gsl_sf_gamma_inc(a, x);
}

/* Unnormalized lower incomplete gamma function:
 *  $\gamma(a,x) = \int_0^x t^{a-1} e^{-t} dt$ */
double lower_gamma(double a, double x) {
    if(x < 0.1)
        return pow(x,a)*(1/a - x/(a+1) + 0.5*x*x/(a+2));
    else
        return gsl_sf_gamma(a) - gsl_sf_gamma_inc(a, x);
}

}
